/*
 * IRC - Internet Relay Chat, ircd/m_sahost.c
 * Copyright (C) 1990 Jarkko Oikarinen and
 *                    University of Oulu, Computing Center
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: m_sahost.c,v 1.8 2004/07/27 23:33:23 commx Exp $
 */

#include "channel.h"
#include "config.h"
#include "client.h"
#include "handlers.h"
#include "hash.h"
#include "ircd.h"
#include "ircd_chattr.h"
#include "ircd_defs.h"
#include "ircd_features.h"
#include "ircd_relay.h"
#include "ircd_reply.h"
#include "ircd_string.h"
#include "ircd_snprintf.h"
#include "ircd_alloc.h"
#include "list.h"
#include "msg.h"
#include "numeric.h"
#include "numnicks.h"
#include "querycmds.h"
#include "send.h"
#include "s_conf.h"
#include "s_user.h"
#include "s_debug.h"
#include "userload.h"
#include "patchlevel.h"

#include <string.h>
#include <ctype.h>
#include <stdlib.h>


int m_sahost(struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{
  struct Client *acptr;
  char *s;
  int legalhost=1;
  struct Membership *chan;

  if(parc<3)
    return(need_more_params(sptr, "SAHOST"));

  if (!HasPriv(sptr, PRIV_SAHOST))
    return send_reply(sptr, ERR_NOPRIVILEGES);

  /* Ignore SAHOST for a user that has quit */
  if(!(acptr = FindClient(parv[1])))
    return send_reply(sptr, ERR_SACOM, "SAHOST", "No such nick");

  /*
   * Check that the given real name is not greater
   * than HOSTLEN.
   */
  if (strlen(parv[2]) > HOSTLEN)
    return send_reply(sptr, ERR_SACOM, "SAHOST", "Host too long");

  for (s = parv[2]; *s; s++)
  {
    if (!IsHostChar(*s))
    {
      legalhost = 0;
      break;
    }
  }

  if (legalhost == 0)
  {
    return send_reply(sptr, ERR_SACOM, "SAHOST", "Illegal characters in hostname");
  }
  /* Send a Quit to all channels the user is on */
  sendcmdto_common_channels_butone(acptr, CMD_QUIT, acptr, ":vHost");
  ircd_strncpy(cli_user(acptr)->vhost, parv[2], HOSTLEN);
  ircd_strncpy(cli_user(acptr)->host, parv[2], HOSTLEN);
  SetAVhost(acptr);

  sendcmdto_serv_butone(sptr, CMD_SAHOST, cptr, "%s %s", parv[1], parv[parc - 1]);
  
  /* Let him rejoin and set modes if he got any */
  for (chan = cli_user(acptr)->channel; chan; chan = chan->next_channel) {
    if (IsZombie(chan)) {
      continue;
    }
    if (!IsChanOp(chan) && !HasVoice(chan) && (chan->channel->mode.mode & CHFL_DELAYED)) {
      SetDelayedJoin(chan);
    } else {
      sendcmdto_channel_butserv_butone(acptr, CMD_JOIN, chan->channel, acptr, "%H", chan->channel);
    }
    if (IsChanOp(chan) && HasVoice(chan)) {
      sendcmdto_channel_butserv_butone(&me, CMD_MODE, chan->channel, acptr, "%H +ov %C %C", chan->channel, acptr, acptr);
    } else if (IsChanOp(chan) || HasVoice(chan)) {
      sendcmdto_channel_butserv_butone(&me, CMD_MODE, chan->channel, acptr, "%H +%c %C", chan->channel, IsChanOp(chan) ? 'o' : 'v', acptr);
    }
  }
  return 0;
}
