/*
 * IRC - Internet Relay Chat, ircd/m_sajoin.c
 * Copyright (C) 1990 Jarkko Oikarinen and
 *                    University of Oulu, Computing Center
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: m_sajoin.c,v 1.4 2003/11/24 16:28:38 sirvulcan Exp $
 */

#include "channel.h"
#include "config.h"
#include "client.h"
#include "handlers.h"
#include "hash.h"
#include "ircd.h"
#include "ircd_chattr.h"
#include "ircd_defs.h"
#include "ircd_features.h"
#include "ircd_relay.h"
#include "ircd_reply.h"
#include "ircd_string.h"
#include "ircd_snprintf.h"
#include "ircd_alloc.h"
#include "list.h"
#include "msg.h"
#include "numeric.h"
#include "numnicks.h"
#include "querycmds.h"
#include "send.h"
#include "s_conf.h"
#include "s_user.h"
#include "s_debug.h"
#include "userload.h"
#include "patchlevel.h"

#include <string.h>
#include <ctype.h>
#include <stdlib.h>


int m_sajoin(struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{
  struct Client *acptr;
  struct Channel *chptr;
  struct JoinBuf join;
  struct JoinBuf create;
  unsigned int flags = 0;
  int j;
  int k = 0;
  char *name;
  char *banreason;

  if (parc < 3)
    return need_more_params(sptr, "SAJOIN");

  if (!HasPriv(sptr, PRIV_SAJOIN))
    return send_reply(sptr, ERR_NOPRIVILEGES);

  if(!(acptr = FindClient(parv[1])))
    return send_reply(sptr, ERR_SACOM, "SAJOIN", "No such nick");

  acptr = FindUser(parv[1]);

  if (IsChannelService(acptr))
    return send_reply(sptr, ERR_NOPRIVILEGES);

  if (!FindChannel(parv[2])) {
    chptr = get_channel(acptr, parv[2], CGT_CREATE);
  } else {
    chptr = get_channel(acptr, parv[2], CGT_NO_CREATE);
  }

  banreason = find_banchan(parv[2]);
  if(banreason)
     return send_reply(sptr, ERR_ILLEGALCHANNAME, parv[2], banreason);

  if (find_member_link(chptr, acptr))
    return 0;

  joinbuf_init(&join, acptr, acptr, JOINBUF_TYPE_JOIN, 0, 0);
  joinbuf_init(&create, acptr, acptr, JOINBUF_TYPE_CREATE, 0, TStime());

  name = chptr->chname;
  clean_channelname(name);

  if (!IsChannelName(name)) { /* bad channel name */
    send_reply(sptr, ERR_NOSUCHCHANNEL, name);
    return 0;
  }

  for (k = 0, j = 0; name[j]; j++)
    if (IsCntrl(name[j]))
      k++;

  if ( k > 0 ) {
    send_reply(sptr, ERR_NOSUCHCHANNEL, name);
    return 0;
  }

  if (chptr->users == 0) {
    flags = CHFL_CHANOP;
  } else {
    flags = CHFL_DEOPPED;
  }

  if (cli_user(acptr)->joined >= feature_int(FEAT_MAXCHANNELSPERUSER)) {
    send_reply(sptr, ERR_TOOMANYCHANNELS, chptr ? chptr->chname : name);
    return 0;
  }

 if (chptr) {
    joinbuf_join(&join, chptr, flags);
 } else if (!(chptr = get_channel(acptr, name, CGT_CREATE)))
     return 0;
   else if (check_target_limit(acptr, chptr, chptr->chname, 1)) {
    sub1_from_channel(chptr); /* created it... */
     return 0;
  } else
      joinbuf_join(&create, chptr, flags);

  if (chptr->topic[0]) {
    send_reply(acptr, RPL_TOPIC, chptr->chname, chptr->topic);
    send_reply(acptr, RPL_TOPICWHOTIME, chptr->chname, chptr->topic_nick,
      chptr->topic_time);
  }

  do_names(acptr, chptr, NAMES_ALL|NAMES_EON); /* send /names list */

  joinbuf_flush(&join); /* must be first, if there's a JOIN 0 */
  joinbuf_flush(&create);

  sendwallto_group_butone(&me, WALL_WALLOPS, 0, "%s is using SAJOIN on %s, joining client into %s", cli_name(sptr), cli_name(acptr), chptr->chname);

  return 0;


}

